/*
 * B-tree application programming interface.
 * Use these functions to create and use a B-tree.
 */

#ifndef API_H
#define API_H

#include "../common/common.h"
#include "../btree/btree.h"
#include "../postings/postings.h"
#include "../docs/docs.h"

/*
 * Open/create the three store files (BTREE_FILE, POSTINGS_FILE, DOCS_FILE).
 */
void db_open();

/*
 * Close the three store files (BTREE_FILE, POSTINGS_FILE, DOCS_FILE).
 */
void db_close();

/*
 * Insert the given document (text file) into the database.
 */
static inline void insert(char* fname) { insert_doc(fname); }

/*
 * Scan all the key entries in the B-tree, applying the function
 * "void f(KeyRecord*)" for each entry.
 */
void scan_tree_leafs(void (*f)(KeyRecord*));

/*
 * Print the given leaf key record, along with the number of docs that contain it.
 * - modified by Frank Andrasco for contiguous postings file.
 */
void print_leaf_key_record(KeyRecord* k);

/*
 * Print the B-tree, page by page, beginning with the Root.
 */
void print_tree_page_order();

/*
 * Print the given page (leaf or non-leaf).
 */
void print_page(long page_num);

/*
 * Recursively print the pages in the B-tree via an in-order traversal,
 * from the given page_num onwards. Call it on the root page with depth 0.
 */
void print_tree_inorder(long page_num, int depth);

/*
 * Search the B-tree for the given key and, if the key is found,
 * print (optionally) the associated postings list.
 */
void search(char* key, int print_flag);

/* Basic key validation */
int validate_key(char* key);

/*
 * Search and print in alphabetical order the distinct keys
 * that are within [key1, key2], i.e., the bounds are included.
 */
void range_search(char* key1, char* key2);

/*
 * Search and print in reverse alphabetical order the distinct keys
 * that are within [key1, key2], i.e., the bounds are included.
 */
void reverse_range_search(char* key1, char* key2);


#endif
