#include "api.h"

/* Forward declarations */
void scan_leaf(PageHdr* p, void (*f)(KeyRecord*));

/*
 * Scan all the key entries in the B-tree, applying the function
 * "void f(KeyRecord*)" for each entry.
 */
void scan_tree_leafs(void (*f)(KeyRecord*)) {
    PageHdr* p = fetch_page(ROOT);
    PageHdr* temp;

    while (!is_leaf(p)) {  /* follow leftmost ptr */
        temp = p;
        p = fetch_page((temp->key_list_ptr)->page_num);
        free_page(temp);
    }

    /* now "p" should point to the alphabetically first page */
    scan_leaf(p, f);
    while ((p->next_leaf_page_num) != NULL_PAGE_NUM) { /* get next valid page */
        temp = p;
        p = fetch_page(temp->next_leaf_page_num);
        free_page(temp);
        scan_leaf(p, f);  /* and process it */
    }
}

/*
 * Scan all the entries of the given leaf page, applying the function "f" for each entry.
 */
void scan_leaf(PageHdr* p, void (*f)(KeyRecord*)) {
    /* traverse all keys in this page */
    KeyRecord* key_list_iterator = p->key_list_ptr;
    while (key_list_iterator != NULL) {
        f(key_list_iterator);
        key_list_iterator = key_list_iterator->next;
    }
}

/*
 * Print the given leaf key record, along with the number of docs that contain it.
 * - modified by Frank Andrasco for contiguous postings file.
 */
void print_leaf_key_record(KeyRecord* k) {
    fseek(fp_postings, k->posting, 0);
    long num_doc_offsets;
    fread(&num_doc_offsets, sizeof(num_doc_offsets), 1, fp_postings);
    printf("%s:%d\n", k->stored_key, (int) num_doc_offsets);
}
