#include "api.h"
#include "../utils/common_words.h"

/*
 * Search the B-tree for the given key and, if the key is found, 
 * print (optionally) the associated postings list.
 */
void search(char* key, int print_flag) {
    if (validate_key(key) == ERROR)
        return;

    /* turn to lower case, for uniformity */
    str_to_lower(key);

    long pptr = search_tree(ROOT, key);

    if (pptr == NON_EXISTENT) {
        printf("key \"%s\": not found\n", key);
    } else {
        if (print_flag)
            print_postings(pptr);
        else
            printf("Found the key!\n");
    }
}

int validate_key(char* key) {
    /* Print an error message if strlen(key) > MAX_WORD_SIZE */
    if (strlen(key) > MAX_WORD_SIZE) {
        printf("ERROR in \"search\": length of key exceeds maximum allowed "
               "so key may be truncated\n");
    }
    if (is_common(key)) {
        printf("\"%s\" is a common word - no searching is done\n", key);
        return ERROR;
    }
    if (is_word_legal(key) == FALSE) {
        printf("\"%s\" is not an alphabetical word - no searching is done\n", key);
        return ERROR;
    }
    return TRUE;
}