#include "btree.h"

double str_to_dbl(char* str);
double str_distance(char* str1, char* str2);

/*
 * This function compares the possible new key with a key stored
 * in the B-Tree. The function returns the value '0' if the keys
 * match, '1' if the new key is lexicographically less than the
 * stored key, and '2' if the new key is lexicographically more
 * than the stored key.
 */
int compare_keys(char* key, char* indexed_key) {
    double diff = str_distance(key, indexed_key);
    if (diff == 0) /* There is a match */
        return 0;
    if (diff < 0) /* New Key < Stored Key */
        return 1;
    if (diff > 0) /* New Key > Stored Key */
        return 2;
}

double str_to_dbl(char* str) {
    double value = 0;
    double denom = 1.0;
    while (*str) {
        value += (*str) / denom;
        str++;
        denom *= 128;
    }
    return value;
}

double str_distance(char* str1, char* str2) {
    return str_to_dbl(str1) - str_to_dbl(str2);
}
