#include "btree.h"

/*
 * This recursive function searches a nonleaf page to find the
 * page number (child page) where the key should be stored. The
 * new key is compared with the keys within the B-Tree page. If
 * the new key is lexicograhically less than or equal to a stored
 * key, the page number of the child page is returned. If all
 * keys within the page have been searched and the new key is
 * lexicographically more than the last key, the page number of
 * the rightmost child is returned.
 */
long find_child_page(PageHdr* page_ptr, KeyRecord* key_list_iterator, char* key, int num_keys) {
    /* Compare the possible new key with key stored in B-Tree */
    char* word = key_list_iterator->stored_key;  /* key stored in B-Tree */
    word[key_list_iterator->key_len] = EOS;

    int result = compare_keys(key, word);

    num_keys = num_keys - 1;

    if (num_keys > 0) {
        if (result == 2) {  /* New key > stored key: keep searching */
            return (find_child_page(page_ptr, key_list_iterator->next, key, num_keys));
        } else  /* New key <= stored key */
            return key_list_iterator->page_num;  /* return left child */

    } else {  /* This is the last key in this page */
        if (result == 1 || result == 0) /* New key <= stored key */
            return key_list_iterator->page_num;  /* return left child */
        else  /* New key > stored key */
            return page_ptr->final_right_page_num;  /* return rightmost child */
    }
}
