#include "btree.h"

/*
 * Recursively searches the subtree starting at the node stored at
 * the given page number for the given key.
 * Returns the offset from the postings file, or ERROR, if not found.
 */
long search_tree(long page_num, char* key) {
    long result;

    PageHdr* page_ptr = fetch_page(page_num);

    if (is_leaf(page_ptr)) {
        result = search_leaf(page_ptr, key);
    }

    /* The root page contains zero keys */
    else if (!is_leaf(page_ptr) && page_ptr->num_keys == 0) {
        /* keys, if any, will be stored in Page #2
         * (which can be leaf or non-leaf). */
        result = search_tree(FIRST_LEAF_PAGE, key);
    }

    else if (!is_leaf(page_ptr) && page_ptr->num_keys > 0) {
        long child_page = find_child_page(page_ptr, page_ptr->key_list_ptr,
                                          key, page_ptr->num_keys);
        result = search_tree(child_page, key);
    }

    free_page(page_ptr);
    return result;
}
