#ifndef POSTINGS_H
#define POSTINGS_H

#include "../common/globals.h"
#include "../common/constants.h"
#include "../common/helpers.h"

/*
 * Write a list of 100 pointers to the beginning of the postings file.
 * These pointers are later used as heads for 100 freelists of increasing size.
 * The value of (-1) is written to indicate that a null list.
 */
void initialize_postings_file();

/*
 * This function prints all the docs pointed to by the postings list at the given offset.
 * Reading the postings list requires only one disk read, since all postings pointers
 * are stored contiguously.
 */
void print_postings(long offset);

/*
 * Add a posting at the given posting_offset pointing to the given doc_offset.
 */
void create_posting(long doc_offset, long* posting_offset);

/*
 * this function returns a file position pointer a free block equal to
 * or greater than the passed size (size). If there is a block of appropriate
 * size in the freelist, it is removed and a pointer to it returned. If not,
 * a dummy block is set up at the end of the file and a pointer to it is returned.
 *
 * The file pointer cannot be assumed to be in the original position after
 * executing this function
 */
long get_record(long size);

/*
 * this function adds the block at the passed postings pointer (pptr) to the
 * freelist of the size (size).
 *
 * The file pointer cannot be assumed to be in the original position after
 * executing this function.
 */
void free_record(long pptr, long size);

/*
 * This function is passed two values: a pointer to an offset in the postings file, and a offset
 * value referring to the textfile. The routines get the record at the postings file offset and
 * checks to see if there is enough space for another text offset. If there is empty offset space
 * available in the same record, the number of text offsets in the record is incremented and
 * re-written to the record and the new text offset is added to the end of the record.
 *
 * If the current postings list is full, all the pointers are read into memory, and the current
 * record is added to a freelist of that size record. A freelist of every size record is kept by
 * keeping the head pointers in the first record of the file.
 *
 * A new free record of the next largest size, if available, is retrieved from the appropriate
 * head pointer at the beginning of the file. If there is no block of the appropriate size, the
 * new record is set up at the end of the file.
 */
void update_postings_file(long* postings_offset, long new_doc_offset);

#endif
